/***************************************************************************
 *
 * File name               :browser.c
 * 
 * Description             :NCS conference manager
 *
 * Authors                 :Tomasz Stachowiak, NPAC
 *
 * Date                    :10-23-96
 *
 ***************************************************************************/
#define XP_UNIX

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <Xm/DialogS.h>
#include <X11/Shell.h>
#include <Xm/MessageB.h>
#include <Xm/List.h>
#include <Xm/ToggleB.h>

/* Motif resource manager (for .uid interface files)*/
#include <Mrm/MrmPublic.h>

#include "launch.c"
#include "../lib/common.h"

typedef struct _ncs_plugin
{
  int active;
  int port;
  char name[50];
  char path[100];
} ncsPlugin;

#define MAX_PLUGINS 10

ncsPlugin plugin[MAX_PLUGINS];
int numPlugins = 0;


XtAppContext    AppContext;
Widget          parent;
Widget          pluginsLabel, membersLabel, pluginsList, membersAccList;
XmStringTable   plugTlist;

String fallbacks[] = { "*fontList: -*-helvetica-bold-r-normal--14-*-*-*-*-*-*-*=charset", NULL };

/* Add member dialog widgets */
Widget	 mainWin, mainForm, textLine, bMemOK, bMemCancel,
         usr_text, host_text;


NCSparticipant invitingPart;
XtWorkProc recworkproc;
XtIntervalId timerId;

int conferenceExists = 0;
int serverUp;

char *information;
Arg wArg[10];
int numArg;
XmString label;
int err;
int dbState = 0;

Boolean recWorkProc();

void start_conference();
void exit_conference();
void partyAccepted();
void partyRejected();
void partyLeft(NCSparticipant *part);
void partyNew(NCSparticipant *part);
void SetMessage(const char *msg);
void createMembersInterface();
void createPluginsInterface();
void serverUsers(void *data);

int init_plugins();

void readPlugins();
void readMembers();
void parseInformation(Widget list, char *info);
/* 
 * Main panel buttons callbacks 
 */
static void add_member(Widget w, XtPointer data1, XtPointer data2);
static void info_plugin(Widget w, XtPointer data1, XtPointer data2);
static void show_plugin(Widget w, XtPointer data1, XtPointer data2);
static void hide_plugin(Widget w, XtPointer data1, XtPointer data2);
static void accept_invitation(Widget w, XtPointer data1, XtPointer data2);
static void reject_invitation(Widget w, XtPointer data1, XtPointer data2);

/* Add dialog buttons callbacks */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data);
void add_cancel(Widget w, XtPointer app_data, XtPointer widget_data);

/* Mrm create callbacks */
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data);
void listform_created(Widget w, XtPointer app_data, XtPointer widget_data);
void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data);
void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data);


/* 
 * Mrm variables 
 */
const String  file_names[2] = { "/CM/main.uid", "/CM/add.uid"};     
String exact_file_names[2];
MrmHierarchy   hierarchy_id;     
MrmType        main_window_class, add_panel_class;

/* main panel widgets */
Widget         main_window, listform, statuslabel, add_but, accept_but, 
               reject_but, show_but, hide_but, info_but, menubar;
Widget         dbIdle, dbSave, dbRestore;

/* add dialog widgets */
Widget         users_list, activeusr_list, add_panel, user_field, host_field;

static MrmRegisterArg cb_list[] = 
{
  { "add_member",            (XtPointer)add_member            },
  { "accept_invitation",     (XtPointer)accept_invitation     },
  { "reject_invitation",     (XtPointer)reject_invitation     },
  { "show_plugin",           (XtPointer)show_plugin           },
  { "hide_plugin",           (XtPointer)hide_plugin           },
  { "info_plugin",           (XtPointer)info_plugin           },
  { "add_usr",               (XtPointer)add_usr               },
  { "delete_usr",            (XtPointer)delete_usr            },
  { "invite_usrs",           (XtPointer)invite_usrs           },
  { "add_cancel",            (XtPointer)add_cancel            },
  { "statuslabel_created",   (XtPointer)statuslabel_created   },
  { "listform_created",      (XtPointer)listform_created      },
  { "add_but_created",       (XtPointer)add_but_created       },
  { "accept_but_created",    (XtPointer)accept_but_created    },
  { "reject_but_created",    (XtPointer)reject_but_created    },
  { "show_but_created",      (XtPointer)show_but_created      },
  { "hide_but_created",      (XtPointer)hide_but_created      },
  { "info_but_created",      (XtPointer)info_but_created      },
  { "users_list_created",    (XtPointer)users_list_created    },
  { "activeusr_list_created",(XtPointer)activeusr_list_created},
  { "user_field_created",    (XtPointer)user_field_created    },
  { "host_field_created",    (XtPointer)host_field_created    }
};


/***********************************************************************
 *
 * Main
 *
 ***********************************************************************/
void file_menu(Widget menu_item, int item_no,
		  void *cbs)
{
   switch(item_no)
     {
        case 0:
	XmListDeleteAllItems(membersAccList);
	conferenceExists = 0;
	ncsSessionReset();
	break;

	case 1:
	exit_conference();
	break;
     }
}

void db_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
        case 0:
	  if(dbState == 1)
	    {
	      ncsSessionSaveClose();
	      XtVaSetValues(dbSave, XmNset, False, NULL);
	    }
	  else if(dbState == 2)
	    {
	      ncsSessionRestoreClose();
	      XtVaSetValues(dbRestore, XmNset, False, NULL);
	    }
	  XtVaSetValues(dbIdle, XmNset, True, NULL);
	  dbState = 0;
	break;

	case 1:
	  if(dbState == 0)
	    {
	      ncsSessionSaveInit();
	      XtVaSetValues(dbSave, XmNset, True, NULL);
	      XtVaSetValues(dbIdle, XmNset, False, NULL);
	      dbState = 1;
	    }
	  else
	    {
	      XtVaSetValues(dbSave, XmNset, False, NULL);
	    }
	  break;

	case 2:
	  if(dbState == 0)
	    {
	      ncsSessionRestoreInit();
	      XtVaSetValues(dbRestore, XmNset, True, NULL);
	      XtVaSetValues(dbIdle, XmNset, False, NULL);
	      dbState = 2;
	    }
	  else
	    {
	      XtVaSetValues(dbRestore, XmNset, False, NULL);
	    }
	  break;
      }
}

main( int argc, char **argv)
{
  char *env;
  int i;
  XmString str1, str2, str3;
  Widget db;

  signal(SIGCHLD, SIG_IGN);

/* Top level widget */
  parent = XtVaAppInitialize(&AppContext, "CE", NULL, 0, 
			     &argc, argv, (ArgList)fallbacks,	
			     XmNwidth,        550,
			     XmNheight,       400,
			     XmNiconName, "NPAC Conference Manager",
			     NULL);
  
  /* Mrm operations */
  MrmInitialize();     
  
  env = getenv("NCS_PATH");
  for(i=0; i<2; i++)
    {
      exact_file_names[i] = malloc(200);
      strcpy(exact_file_names[i], env);
      strcat(exact_file_names[i], file_names[i]);
    }

  MrmOpenHierarchyPerDisplay(XtDisplay(parent), 
			     2, 
			     exact_file_names,              
			     (MrmOsOpenParamPtr *) NULL, 
			     &hierarchy_id);
  
  for(i=0; i<2; i++)
    {
      free(exact_file_names[i]);
    }

  if (MrmRegisterNames(cb_list, 
		       (MrmCount)XtNumber(cb_list)) != MrmSUCCESS)
    {
      fprintf(stderr, "Unable to register callbacks\n");
      fflush(stderr);
    }
  
  if (MrmFetchWidget(hierarchy_id, "simpleWindow",
		     parent, &main_window, &main_window_class)
      != MrmSUCCESS) 
    {
      fprintf(stderr, "Unable to fetch top-level widget.\n");
      fflush(stderr);
    }      
  
 /* Menu bar */
  str1 = XmStringCreateSimple("File");
  str2 = XmStringCreateSimple("Database");
  menubar = XmVaCreateSimpleMenuBar(main_window, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    XmVaCASCADEBUTTON, str2, 'D',
	    NULL);

  XmStringFree(str1);
  XmStringFree(str2);
				    
/* File menu, callback: file_menu() */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  str3 = XmStringCreateSimple("Reset session");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str3, 'R', NULL, NULL,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  XmStringFree(str3);

  db = XmVaCreateSimplePulldownMenu(menubar, "db_menu", 1, db_menu,
	    NULL);
  dbIdle = XtVaCreateManagedWidget("Normal", 
				   xmToggleButtonWidgetClass, db, 
				   XmNset, True,
				   NULL); 
  XtAddCallback(dbIdle, XmNvalueChangedCallback, db_menu, 0); 
  dbSave = XtVaCreateManagedWidget("Save", 
				   xmToggleButtonWidgetClass, db, NULL); 
  XtAddCallback(dbSave, XmNvalueChangedCallback, db_menu, 1); 
  dbRestore = XtVaCreateManagedWidget("Restore", 
				   xmToggleButtonWidgetClass, db, NULL); 
  XtAddCallback(dbRestore, XmNvalueChangedCallback, db_menu, 2); 

  XtManageChild(db);

  XtManageChild(menubar);
  XtManageChild(main_window);
  XtRealizeWidget(parent);
  
  timerId = XtAppAddTimeOut(AppContext,
			    100,
			    start_conference,
			    NULL); 
  
	
  XtAppMainLoop(AppContext);
}

/*********************************************************************
 *
 * Mrm create callbacks
 *
 *********************************************************************/
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  statuslabel = w;
  SetMessage("Initializing Conference Engine...");
}

void listform_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  listform = w;
}

void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  add_but = w;
}

void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  accept_but = w;
  XtSetSensitive(w, False);
}

void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  reject_but = w;
  XtSetSensitive(w, False);
}

void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  show_but = w;
  XtSetSensitive(w, False);
}
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  hide_but = w;
  XtSetSensitive(w, False);
}


void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  info_but = w;
  XtSetSensitive(w, False);
}

void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  users_list = w;
}

void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  activeusr_list = w;
}

void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  user_field = w;
}

void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  host_field = w;
}



/*************************************************************************
  *
  *  Callbacks invoked by buttons
  *
  ************************************************************************/
showMessage(Widget w, char *msg)
{
  Widget dialog;
  XmString text;
  XmString title;

  text = XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET);
  title = XmStringCreateSimple("Message");
  dialog = XmCreateMessageDialog(w, "message", NULL, 0);
  XtVaSetValues(dialog,
		XmNmessageString, text,
		XmNdialogType, XmDIALOG_INFORMATION,
		XmNdialogTitle, title,
		NULL);
  XmStringFree(text);
  XmStringFree(title);

  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));

  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
}


/***********************************
 * 
 * Add member dialog
 * 
 **********************************/
void add_member(Widget w, XtPointer data1, XtPointer data2)
{
  Widget usr_lab, host_lab;

  ncsServerUsers();

  mainWin = XtVaCreatePopupShell( "mainWin",
				 xmDialogShellWidgetClass,
				 parent,
				 XmNx, 370,
				 XmNy, 300,
				 XmNwidth, 500,
				 XmNheight, 470,
				 XmNtitle, "Add members",
				 NULL );

  

  if (MrmFetchWidget(hierarchy_id, "addWindow",
		     mainWin, &add_panel, &add_panel_class)
      != MrmSUCCESS) 
    {
      fprintf(stderr, "Unable to fetch top-level 'add' widget.\n");
      fflush(stderr);
    }      

  XtManageChild(add_panel);

  XtRealizeWidget(mainWin);

  XtPopup(mainWin, XtGrabNone);

  parseInformation(users_list, information);

  return;
}

/*
 * Add member dialog callbacks 
 */

/* Cancel callback */
void add_cancel(Widget w, XtPointer app_data, XtPointer widget_data)
{
  XtDestroyWidget(mainWin);
}

/* Activate user (add button) */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;
  char *user;
  char *host;
  char malpa = '@';
  char addr[200];
  XmString address;

  XtVaGetValues(users_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(users_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      if(!XmListItemExists(activeusr_list, sel_item[0]))
	XmListAddItem(activeusr_list, sel_item[0], 0);
    }

  XtVaGetValues(user_field,
		XmNvalue, &user,
		NULL);
  XtVaGetValues(host_field,
		XmNvalue, &host,
		NULL);
  if((strlen(user) > 0) && (strlen(host) > 0))
    {
      strcpy(addr, user);
      strcat(addr, &malpa);
      strcpy(&addr[strlen(user) + 1], host);
      address = XmStringCreateSimple(addr);
      if(!XmListItemExists(activeusr_list, address))
	{
	  XmListAddItem(activeusr_list, address, 0);
	  XmTextFieldSetString(user_field, "");
	  XmTextFieldSetString(host_field, "");
	}
      XmStringFree(address);
    }
  
}
		
/* Unactivate user (delete button) */
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;

  XtVaGetValues(activeusr_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(activeusr_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      XmListDeleteItem(activeusr_list, sel_item[0]);
    }  
}

/* Invite active users */
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int item_no;
  XmStringTable items;
  char *item;
  int i;
  NCSparticipant part;

  XtVaGetValues(activeusr_list,
		XmNitemCount, &item_no,
		NULL);
  if(item_no > 0)
    {
      if(!conferenceExists)
	{
	  ncsSessionCreate();
	  conferenceExists = 1;
	}

      XtVaGetValues(activeusr_list,
		    XmNitems, &items,
		    NULL);
      for(i=0; i<item_no; i++)
	{
	  XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &item);
	  parseIPaddress(item, part.user, part.host);
	  XtFree(item);
	  ncsSessionAddMember(part.user, part.host);
	}  
    }
  XtDestroyWidget(mainWin);
}


/********************* Add members dialog END ***************************/


/************************************************************************
 *
 * Callbacks regarding plugins management 
 * 
 ************************************************************************/
void info_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;
  char *text;
  int len;
  char cport[10];

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      len = strlen(plugin[pos[0][0]-1].path) 
	+ strlen(plugin[pos[0][0]-1].name) + 100;
      text = malloc(len);
      strcpy(text, "PLUGIN INFO\nName: ");
      strcat(text, plugin[pos[0][0]-1].name);
      strcat(text, "\nPort: ");
      sprintf(cport, "%d", plugin[pos[0][0]-1].port);
      strcat(text, cport);
      strcat(text, "\nPath: ");
      strcat(text, plugin[pos[0][0]-1].path);

      showMessage(parent, text);

      free(text);
    }
}

void show_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;
  char class[30], tmp[10];

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      if(!plugin[pos[0][0]-1].active)
	{
	  if(strncmp(plugin[pos[0][0]-1].path, "java", 4) == 0)
	    {
	      err = startJava(plugin[pos[0][0]-1].path);
	    }
	  else
	    {
	      err = startProg(plugin[pos[0][0]-1].path, "", "");	    
	    }

	  if(err  < 0)
	     {
	       SetMessage("Can not start plugin");
	       return;
	     }
	  else
	    {
	      ncsSessionStartApp(plugin[pos[0][0]-1].port);
	      plugin[pos[0][0]-1].active = 1;
	    }
	}
    }
}

void hide_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  int **pos;
  int count;

  if(XmListGetSelectedPos(pluginsList, pos, &count))
    {
      if(plugin[pos[0][0]-1].active)
	{
	  ncsSessionSendToApp(plugin[pos[0][0]-1].port, indLocalAppQuit, NULL, 0);
	  plugin[pos[0][0]-1].active = 0;
	}
    }
}

void accept_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  if(!conferenceExists)
    {
      conferenceExists = 1;
    }
  ncsSessionAcceptInvitation(&invitingPart);
  SetMessage("Invitation to teleconference has been accepted!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
    
  return;
}

void reject_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  ncsSessionRejectInvitation(&invitingPart);
  SetMessage("Invitation to teleconference has been rejected!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
  return;
}


/***************************************************************************
 * 
 * NCS Events callbacks
 * 
 ***************************************************************************/
void startApp(void *data)
{
  int *port;
  int i;
  char class[30], tmp[10];

  port = (int *)data;
  for(i=0; i<numPlugins; i++)
    {
      if(plugin[i].port == *port)
	{
	  if(!plugin[i].active)
	    {
	      if(strncmp(plugin[i].path, "java", 4) == 0)
		{
		  err = startJava(plugin[i].path);
		}
	      else
		{
		  err = startProg(plugin[i].path, "", "");	    
		}
	      
	      if(err  < 0)
		{
		  SetMessage("Can not start plugin");
		  return;
		}
	      else
		plugin[i].active = 1;
	    }
	}
    }
}

void partyTimeout(NCSparticipant *part)
{
  char text[100];

  sprintf(text, "User %s@%s\n not present!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyConnected(NCSparticipant *part)
{
  char text[100];

  sprintf(text, "User %s@%s\n received invitation!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyAccepted(NCSparticipant *part)
{
  char text[100];

  sprintf(text, "User %s@%s\n accepted invitation!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyRejected(NCSparticipant *part)
{
  char text[100];

  sprintf(text, "User %s@%s\n rejected invitation!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyLeft(NCSparticipant *part)
{
  char text[100];
  XmString str;

  sprintf(text, "%s@%s", part->user, part->host);
  str = XmStringCreateSimple(text);
  if(XmListItemExists(membersAccList, str))
    XmListDeleteItem(membersAccList, str);
  XmStringFree(str);

  sprintf(text, "User %s@%s\n left conference!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyNew(NCSparticipant *part)
{
  char text[100];
  XmString str;

  sprintf(text, "%s@%s", part->user, part->host);
  str = XmStringCreateSimple(text);
  XmListAddItem(membersAccList, str, 0);
  XmStringFree(str);

  sprintf(text, "User %s@%s\n joined conference!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void invited(NCSparticipant *part)
{
  char text[100];
  int res;

  sprintf(text, "RECEIVED INVITATION!\n from: %s@%s\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);

  XtSetSensitive(accept_but, True);
  XtSetSensitive(reject_but, True);
  SetMessage("Press ACCEPT or REJECT");

  memcpy(&invitingPart, part, sizeof(NCSparticipant));
  
  return;
}

void SetMessage(const char *msg)
{
  XmString str=XmStringCreateSimple((char*)msg);
  XtVaSetValues(statuslabel,
		XmNlabelString, str,
		NULL);
  XmStringFree(str);
  return;
}

/***************************************************************************

  Definitions of OpenDVE-specific functions

***************************************************************************/  

void start_conference(void)
{ 
  int err;
  char *ncs_path;
  char path[100];

  signal(SIGCHLD, SIG_IGN);

  ncs_path = getenv("NCS_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/Daemon/NCSengine");
  SetMessage("Initializing Conference Engine...");
  if ((err = check_process(path, "NCSengine")) < 0)
    {
      SetMessage("Can not initialize NCS... Aborted...");
      return;
    }

  if(!ncsSessionInitialize())
    {
      fprintf(stderr, "NCS session already exists...Exiting...\n");
      exit(-1);
    }

  ncsSessionAddCallback(indLocalSessionInvited, invited);
  ncsSessionAddCallback(indLocalSessionAddMember, partyNew);
  ncsSessionAddCallback(indLocalSessionDeleteMember, partyLeft);
  ncsSessionAddCallback(indLocalSessionAccept, partyAccepted);
  ncsSessionAddCallback(indLocalSessionReject, partyRejected);
  ncsSessionAddCallback(indLocalSessionTimeout, partyTimeout);
  ncsSessionAddCallback(indLocalSessionConnected, partyConnected);
  ncsSessionAddCallback(indLocalSessionStartApp, startApp);
  ncsSessionAddCallback(indServerUsers, serverUsers);

  /* NCS start working procedure reading messages */
  recworkproc = XtAppAddWorkProc(AppContext, recWorkProc, NULL);

  SetMessage("Connected to Conference Engine...");
  fflush(stdout);
  SetMessage("Initializing conferencing panel...");

  /*readMembers();*/
  serverUp = 0;
  ncsServerUsers();
  readPlugins();

  createPluginsInterface();  
  createMembersInterface();
  
  SetMessage("Conferencing panel ready to use.");
}


void exit_conference(void)
{
  int i;

  XtRemoveWorkProc(recworkproc);
  for(i=0; i<numPlugins; i++)
    {
      if(plugin[i].active)
	{
	  ncsSessionSendToApp(plugin[i].port, indLocalAppQuit, NULL, 0);
	  plugin[i].active = 0;
	}
    }
  ncsSessionQuit();
  exit(0);
}


/**********************************************************************
 *
 * Plugins list procedures
 *
 *********************************************************************/
void pluginSelected(Widget w, XtPointer data1, XtPointer data2)
{
  XmListCallbackStruct *cbs=(XmListCallbackStruct *)data2;

/*  printf("Selected item: %d\n", cbs->item_position-1);
  printf("Number of items: %d\n", cbs->selected_item_count);
  fflush(stdout);*/
  if (cbs->selected_item_count > 0)
  {
    XtSetSensitive(show_but, True);
    XtSetSensitive(hide_but, True);
    XtSetSensitive(info_but, True);
  }
  else
  {
    XtSetSensitive(show_but, False);
    XtSetSensitive(hide_but, False);
    XtSetSensitive(info_but, False);
  }
  
  return;
}

void createPluginsInterface()
{
  int i;
  XmString label;

  label=XmStringCreateLocalized("Available plugins:");
  pluginsLabel=XtVaCreateManagedWidget("pluginsLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 10,
				       XmNy, 10,
				       NULL);


  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 10); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 220); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  pluginsList = XmCreateScrolledList(listform, "Plugins", wArg, numArg);

  plugTlist=NULL;
  XtVaSetValues(pluginsList,
	        XmNitems, plugTlist,
		XmNitemCount, 0,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
  plugTlist=(XmStringTable)XtMalloc(numPlugins * sizeof(XmString *));
  for(i=0; i<numPlugins;i++)
  {
    plugTlist[i]=XmStringCreateLocalized(plugin[i].name);
  }
  
  XtVaSetValues(pluginsList,
	        XmNitems, plugTlist,
		XmNitemCount, numPlugins,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
  
  XtAddCallback(pluginsList, XmNsingleSelectionCallback, pluginSelected, NULL);
  XtManageChild(pluginsList);
  
  
  for(i=0; i<numPlugins;i++)
  {
    XmStringFree(plugTlist[i]);
  }
  
  XtFree(plugTlist);

  return;
}



/************************************************************************
 *
 * Members list procedures
 *
 ************************************************************************/
void createMembersInterface()
{
  int i;

  /* list of members which accepted invitation */
  label=XmStringCreateLocalized("Active members:");
  membersLabel=XtVaCreateManagedWidget("membersLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 240,
				       XmNy, 10,
				       NULL);

  
  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 240); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 250); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  membersAccList = XmCreateScrolledList(listform, "membersAcc",
					       wArg, numArg);

  XtVaSetValues(membersAccList,
		XmNitemCount, 0/*appData.numInv*/,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmMULTIPLE_SELECT,
		NULL);
					   		
  XtManageChild(membersAccList);
  
}


void serverUsers(void *data)
{
  ServerData *users;
  int pos=0;
  int i;

  users = (ServerData *)data;
  information = malloc(users->numMembers * sizeof(NCSparticipant));
  pos += sprintf(information, "%d\n", users->numMembers);
  for(i=0; i<users->numMembers; i++)
    {
      pos += sprintf(&information[pos], "%s@%s\n", users->parts[i].user,
	      users->parts[i].host);
    }
  
  serverUp = 1;
}


/*
 * function adding members from http file to adding list
 */
void readMembers()
{
  int size;
  FILE *file;

  file = fopen("members.ncs", "r");
  fseek(file, 0, SEEK_END);
  size = ftell(file);
  information = malloc(size);
  fseek(file, 0, SEEK_SET);
  fread(information, size, 1, file);
  fclose(file);
}


void parseInformation(Widget list, char *info)
{
  int pos = 0;
  char user[100];
  XmString xmstring;
  int number, i;

  if(serverUp)
    {
      sscanf(info, "%d\n", &number);
      sscanf(info, "%s\n", user);
      pos = strlen(user) + 1;
      for(i=0; i<number; i++)
	{
	  sscanf(&info[pos], "%s\n", user);
	  pos += strlen(user) + 1;
	  xmstring = XmStringCreateSimple(user);
	  XmListAddItem(list, xmstring, 0);
	  XmStringFree(xmstring);
	}
    }
}

void readPlugins()
{
  FILE *file;
  char *ncs_path;
  char path[100];

  ncs_path = getenv("NCS_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/CM/plugins.ncs");
  file = fopen(path, "r");

  while(!feof(file))
    {
      fscanf(file, "%d\n", &plugin[numPlugins].port); 
      fgets(plugin[numPlugins].name, 50, file);
      strcpy(plugin[numPlugins].path, ncs_path);
      fgets(path, 100, file);
      if(strncmp(path, "java", 4) == 0)
	{
	  strcpy(plugin[numPlugins].path, path);
	}
      else
	strcat(plugin[numPlugins].path, path);
      plugin[numPlugins].path[strlen(plugin[numPlugins].path) - 1] = '\0';
      plugin[numPlugins].name[strlen(plugin[numPlugins].name) - 1] = '\0';
      plugin[numPlugins].active = 0;
      numPlugins++;
    }
  fclose(file);
}

/***************************************************************************
 *
 * Main control messages processing procedure
 *
 ***************************************************************************/
Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  int sockfd;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;
  sockfd = ncsSessionGetFD(&set);
  if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
    {
      ncsProcessIncoming(&set);
    }

  return False;
}










